@@ -12,7 +12,7 @@ import collections
 from functools import wraps
 import logging
 
-__version__ = "0.2.1"
+__version__ = "0.2.2"
 
 # stuff for Python 3 compatibility
 try:
@@ -24,12 +24,12 @@ try:
     reduce
 except NameError:
     from functools import reduce
-    
+
 try:
     basestring
 except NameError:
     basestring = str
-    
+
 
 logger = logging.getLogger("lazyarray")
 
@@ -57,6 +57,39 @@ def requires_shape(meth):
     return wrapped_meth
 
 
+def full_address(addr, full_shape):
+    if not isinstance(addr, tuple):
+        addr = (addr,)
+    if len(addr) < len(full_shape):
+        full_addr = [slice(None)] * len(full_shape)
+        for i, val in enumerate(addr):
+            full_addr[i] = val
+        addr = full_addr
+    return addr
+
+
+def partial_shape(addr, full_shape):
+    """
+    Calculate the size of the sub-array represented by `addr`
+    """
+    def size(x, max):
+        if isinstance(x, (int, long)):
+            return 1
+        elif isinstance(x, slice):
+            return 1 + ((x.stop or max) - (x.start or 0) - 1) // (x.step or 1)
+        elif isinstance(x, collections.Sized):
+            if hasattr(x, 'dtype') and x.dtype == bool:
+                return x.sum()
+            else:
+                return len(x)
+    addr = full_address(addr, full_shape)
+    shape = [size(x, max) for (x, max) in zip(addr, full_shape)]
+    if shape == [1] or shape == [1, 1]:
+        return [1]
+    else:
+        return [x for x in shape if x > 1] # remove empty dimensions
+
+
 def reverse(func):
     """Given a function f(a, b), returns f(b, a)"""
     @wraps(func)
@@ -199,6 +232,7 @@ class larray(object):
     @property
     @requires_shape
     def size(self):
+        """Total number of elements in the array."""
         return reduce(operator.mul, self._shape)
 
     @property
@@ -213,22 +247,7 @@ class larray(object):
         """
         Calculate the size of the sub-array represented by `addr`
         """
-        def size(x, max):
-            if isinstance(x, (int, long)):
-                return 1
-            elif isinstance(x, slice):
-                return 1 + ((x.stop or max) - (x.start or 0) - 1) // (x.step or 1)
-            elif isinstance(x, collections.Sized):
-                if hasattr(x, 'dtype') and x.dtype == bool:
-                    return x.sum()
-                else:
-                    return len(x)
-        addr = self._full_address(addr)
-        shape = [size(x, max) for (x, max) in zip(addr, self._shape)]
-        if shape == [1] or shape == [1, 1]:
-            return [1]
-        else:
-            return [x for x in shape if x > 1] # remove empty dimensions
+        return partial_shape(addr, self._shape)
 
     def _homogeneous_array(self, addr):
         self.check_bounds(addr)
@@ -239,14 +258,7 @@ class larray(object):
             return numpy.ones(shape, type(self.base_value))
 
     def _full_address(self, addr):
-        if not isinstance(addr, tuple):
-            addr = (addr,)
-        if len(addr) < len(self.shape):
-            full_addr = [slice(None)] * len(self._shape)
-            for i, val in enumerate(addr):
-                full_addr[i] = val
-            addr = full_addr
-        return addr
+        return full_address(addr, self._shape)
 
     def _array_indices(self, addr):
         self.check_bounds(addr)
@@ -262,7 +274,7 @@ class larray(object):
                 if hasattr(x, 'dtype') and x.dtype == bool:
                     return numpy.arange(max)[x]
                 else:
-                    return x
+                    return numpy.array(x)
         addr = self._full_address(addr)
         indices = [axis_indices(x, max) for (x, max) in zip(addr, self._shape)]
         if len(indices) == 1:
@@ -278,6 +290,12 @@ class larray(object):
 
     @requires_shape
     def __getitem__(self, addr):
+        """
+        Return one or more items from the array, as for NumPy arrays.
+
+        `addr` may be a single integer, a slice, a NumPy boolean array or a
+        NumPy integer array.
+        """
         return self._partially_evaluate(addr, simplify=False)
 
     def _partially_evaluate(self, addr, simplify=False):
@@ -294,6 +312,8 @@ class larray(object):
         elif callable(self.base_value):
             indices = self._array_indices(addr)
             base_val = self.base_value(*indices)
+            if isinstance(base_val, numpy.ndarray) and base_val.shape == (1,):
+                base_val = base_val[0]
         elif isinstance(self.base_value, VectorizedIterable):
             partial_shape = self._partial_shape(addr)
             if partial_shape:
@@ -301,7 +321,9 @@ class larray(object):
             else:
                 n = 0
             base_val = self.base_value.next(n) # note that the array contents will depend on the order of access to elements
-            if partial_shape and base_val.shape != partial_shape:
+            if n == 1:
+                base_val = base_val[0]
+            elif partial_shape and base_val.shape != partial_shape:
                 base_val = base_val.reshape(partial_shape)
         elif isinstance(self.base_value, collections.Iterator):
             raise NotImplementedError("coming soon...")
@@ -382,7 +404,7 @@ class larray(object):
         elif isinstance(self.base_value, numpy.ndarray):
             x = self.base_value
         elif callable(self.base_value):
-            x = numpy.array(numpy.fromfunction(self.base_value, shape=self._shape), dtype=self.dtype)
+            x = numpy.array(numpy.fromfunction(self.base_value, shape=self._shape, dtype=int), dtype=self.dtype)
         elif isinstance(self.base_value, VectorizedIterable):
             x = self.base_value.next(self.size)
             if x.shape != self._shape:
@@ -395,6 +417,21 @@ class larray(object):
             raise ValueError("invalid base value for array")
         return self._apply_operations(x, simplify=simplify)
 
+    def __call__(self, arg):
+        if callable(self.base_value):
+            if callable(arg):
+                new_map = larray(arg)
+            elif isinstance(arg, lazyarray):
+                new_map = deepcopy(arg)
+            else:
+                raise Exception("Argument must be either callable or an larray.")
+            new_map.operations.append((self.base_value, None))
+            new_map.operations.extend(self.operations)
+            return new_map
+        else:
+            raise Exception("larray is not callable")
+
+
     __iadd__ = lazy_inplace_operation('add')
     __isub__ = lazy_inplace_operation('sub')
     __imul__ = lazy_inplace_operation('mul')
